@@ -12,13 +12,11 @@
 #' @return A data frame containing the cleaned and merged dataset.
 #'
 #' @examples
-#' \donttest{
-#'   # Example usage (ensure that 'inst/extdata' contains the appropriate files,
-#'   # or modify this example to use your own data)
-#'   microbiome_data <- system.file("extdata", "microbiome.csv", package = "mbX")
-#'   metadata <- system.file("extdata", "metadata.csv", package = "mbX")
-#'   if (nzchar(microbiome_data) && nzchar(metadata)) {
-#'     cleaned_data <- ezclean(microbiome_data, metadata, "genus")
+#' \dontrun{
+#'   mb  <- system.file("extdata", "microbiome.csv", package = "mbX")
+#'   md  <- system.file("extdata", "metadata.csv",   package = "mbX")
+#'   if (nzchar(mb) && nzchar(md)) {
+#'     cleaned_data <- ezclean(mb, md, "g")
 #'     head(cleaned_data)
 #'   } else {
 #'     message("Sample data files not found.")
@@ -65,6 +63,27 @@ ezclean <- function(microbiome_data, metadata, level = "d") {
     return("Please check the first header of the metadata for file format correction.")
   }
   
+  # — sanitize metadata column names & values —
+  # replace spaces in column names
+  colnames(metadata_df) <- gsub("\\s+", "_", colnames(metadata_df))
+  
+  # for any character columns, replace spaces in the values
+  char_cols <- sapply(metadata_df, is.character)
+  metadata_df[ , char_cols] <- lapply(
+    metadata_df[ , char_cols, drop = FALSE],
+    function(x) gsub("\\s+", "_", x)
+  )
+  
+  # for any factor columns, replace spaces in the levels
+  factor_cols <- sapply(metadata_df, is.factor)
+  metadata_df[ , factor_cols] <- lapply(
+    metadata_df[ , factor_cols, drop = FALSE],
+    function(x) factor(gsub("\\s+", "_", as.character(x)))
+  )
+  
+  
+  
+  
   
   
   # Read microbiome data
@@ -79,6 +98,16 @@ ezclean <- function(microbiome_data, metadata, level = "d") {
     return("The microbiome file is not in a supported format. Please use txt, csv, xls, or xlsx.")
   }
   
+  
+  # — sanitize first column values — 
+  # replace spaces with underscores in whatever the first column is  
+  first_col <- names(microbiome_df)[1]  
+  microbiome_df[[first_col]] <- gsub("\\s+", "_", as.character(microbiome_df[[first_col]]))
+  
+  
+  # grab just the basename (no path, no extension)
+  microbiome_base <- tools::file_path_sans_ext(basename(microbiome_data))
+  
   # Define the levels map with all variations
   levels_map <- c(
     "domain" = "d__", "Domain" = "d__", "DOMAIN" = "d__", "D" = "d__", "d" = "d__",
@@ -268,7 +297,7 @@ ezclean <- function(microbiome_data, metadata, level = "d") {
         if (found_value != "") {
           blank_counter <<- blank_counter + 1
           return(paste0("unidentified_", level_mapping[level_value], "_", blank_counter,
-                        "_at_", found_value, "_", level_mapping[found_marker]))
+                        "_from_", found_value, "_", level_mapping[found_marker]))
         } else {
           return("")
         }
@@ -535,30 +564,28 @@ ezclean <- function(microbiome_data, metadata, level = "d") {
   
   
   
-  #ADDING THE METEDATACOLUMNS TO THE CLEANED DATA SET
-  # ---- Additional Metadata Merge Block Start ----
+  # ---- Revised Metadata Merge Block Start ----
   # Read the microbiome_ezy_12.xlsx file (with header)
   df_ezy12 <- openxlsx::read.xlsx("microbiome_ezy_12.xlsx", colNames = TRUE)
   
-  # Use the metadata_df that was loaded earlier in the function.
-  # Assume that the first column in metadata_df is the key to match.
-  # Extract the remaining metadata columns (i.e., all except the first)
+  # Extract only the first column (sample-ID) and the rest of the microbiome columns
+  first_col   <- df_ezy12[, 1, drop = FALSE]
+  micro_cols  <- df_ezy12[, -1, drop = FALSE]
+  
+  # Prepare metadata exactly as before:
   meta_remaining <- metadata_df[, -1, drop = FALSE]
-  
-  # Extract the key values from metadata_df and df_ezy12
-  meta_key <- metadata_df[[1]]
-  df_key   <- df_ezy12[[1]]
-  
-  # For each row in df_ezy12, find the matching row in metadata_df
-  # and extract the corresponding remaining metadata columns.
+  meta_key       <- metadata_df[[1]]
+  df_key         <- df_ezy12[[1]]
   appended_metadata <- meta_remaining[match(df_key, meta_key), , drop = FALSE]
   
-  # Combine the microbiome_ezy_12 data with the appended metadata columns
-  df_ezy13 <- cbind(df_ezy12, appended_metadata)
+  # Now bind them in the desired order:
+  #  (1) sample-ID, (2) metadata columns, (3) microbiome columns
+  df_ezy13 <- cbind(first_col, appended_metadata, micro_cols)
   
   # Save the combined data as microbiome_ezy_13.xlsx with headers
   openxlsx::write.xlsx(df_ezy13, "microbiome_ezy_13.xlsx", rowNames = FALSE)
-  # ---- Additional Metadata Merge Block End ----
+  # ---- Revised Metadata Merge Block End ----
+  
   
   
   
@@ -623,23 +650,43 @@ ezclean <- function(microbiome_data, metadata, level = "d") {
   
   ####FINAL NAME RETURN
   
-  
   # ---- Final Return Block ----
   final_names <- list(
-    "d__" = "mbX_cleaned_domains_or_kingdom.xlsx",
-    "p__" = "mbX_cleaned_phylum.xlsx",
-    "c__" = "mbX_cleaned_classes.xlsx",
-    "o__" = "mbX_cleaned_orders.xlsx",
-    "f__" = "mbX_cleaned_families.xlsx",
-    "g__" = "mbX_cleaned_genera.xlsx",
-    "s__" = "mbX_cleaned_species.xlsx"
+    "d__" = paste0("mbX_cleaned_domains_or_kingdoms_", microbiome_base, ".xlsx"),
+    "p__" = paste0("mbX_cleaned_phyla_",                     microbiome_base, ".xlsx"),
+    "c__" = paste0("mbX_cleaned_classes_",                    microbiome_base, ".xlsx"),
+    "o__" = paste0("mbX_cleaned_orders_",                     microbiome_base, ".xlsx"),
+    "f__" = paste0("mbX_cleaned_families_",                   microbiome_base, ".xlsx"),
+    "g__" = paste0("mbX_cleaned_genera_",                     microbiome_base, ".xlsx"),
+    "s__" = paste0("mbX_cleaned_species_",                    microbiome_base, ".xlsx")
   )
   
-  final_file_name <- final_names[[level_value]]
-  file.rename("microbiome_ezy_13.xlsx", final_file_name)
   
-  return(final_file_name)
-  cat("new ho hai")
+  
+  
+  
+  final_file_name <- final_names[[level_value]]
+  #file.rename("microbiome_ezy_13.xlsx", final_file_name)
+  
+  dir_name_ezclean <- tools::file_path_sans_ext(final_file_name)
+  
+  # my new dir ccreate folder if needed:
+  if (!dir.exists(dir_name_ezclean)) {
+    dir.create(dir_name_ezclean)
+  }
+  
+  # 4) Move (rename) the file into that folder:
+  new_path <- file.path(dir_name_ezclean, final_file_name)
+  file.rename("microbiome_ezy_13.xlsx", new_path)
+  
+  # 5) Return the full path of the moved file:
+  cat("Done with the cleaning, your cleaned file is here: ")
+  return(new_path)
+  
+  
+  message('Please cite us: "Lamichhane, U., & Lourenco, J. (2025). mbX: An R Package for Streamlined Microbiome Analysis. Stats, 8(2), 44."')
+  message("DOI to paper: https://doi.org/10.3390/stats8020044")
+  
 }
 
 
@@ -667,22 +714,32 @@ ezclean <- function(microbiome_data, metadata, level = "d") {
 #'        threshold, but not both.
 #' @param threshold An optional numeric value indicating the minimum threshold value; taxa below this
 #'        threshold will be grouped into an "Other" category.
+#' @param flip Logical. If `TRUE`, the order of the stacks is reversed.
 #'
 #' @return A ggplot object containing the visualization.
 #'
 #' @examples
 #' \donttest{
-#'   # Example usage (ensure that 'inst/extdata' contains the appropriate files):
-#'   microbiome_data <- system.file("extdata", "microbiome.csv", package = "mbX")
-#'   metadata <- system.file("extdata", "metadata.csv", package = "mbX")
-#'   plot_obj <- ezviz(microbiome_data, metadata, "genus", "sample_type", top_taxa = 20)
-#'   print(plot_obj)
+#' mb  <- system.file("extdata", "microbiome.csv", package = "mbX")
+#' md  <- system.file("extdata", "metadata.csv",   package = "mbX")
+#' plot_obj <- ezviz(
+#'   microbiome_data = mb,
+#'   metadata        = md,
+#'   level           = "genus",
+#'   selected_metadata    = "sample_type",
+#'   top_taxa        = 20,
+#'   flip            = FALSE
+#' )
+#' print(plot_obj)
 #' }
+
 #'
 #' @import ggplot2
 #' @importFrom dplyr filter
 #' @export
-ezviz <- function(microbiome_data, metadata, level, selected_metadata, top_taxa = NULL, threshold = NULL) {
+ezviz <- function(microbiome_data, metadata, level, selected_metadata,
+                  top_taxa = NULL, threshold = NULL,
+                  flip = FALSE) {
   # Check the file extension for microbiome_data
   microbiome_ext <- tools::file_ext(microbiome_data)
   if (!microbiome_ext %in% c("csv", "xls", "xlsx")) {
@@ -728,6 +785,27 @@ ezviz <- function(microbiome_data, metadata, level, selected_metadata, top_taxa
     return("The selected metadata is either not in the metadata or not a categorical value")
   }
   
+  
+  # — sanitize metadata column names & values —
+  # replace spaces in column names
+  colnames(metadata_df) <- gsub("\\s+", "_", colnames(metadata_df))
+  
+  # for any character columns, replace spaces in the values
+  char_cols <- sapply(metadata_df, is.character)
+  metadata_df[ , char_cols] <- lapply(
+    metadata_df[ , char_cols, drop = FALSE],
+    function(x) gsub("\\s+", "_", x)
+  )
+  
+  # for any factor columns, replace spaces in the levels
+  factor_cols <- sapply(metadata_df, is.factor)
+  metadata_df[ , factor_cols] <- lapply(
+    metadata_df[ , factor_cols, drop = FALSE],
+    function(x) factor(gsub("\\s+", "_", as.character(x)))
+  )
+  
+  
+  
   # Read microbiome data
   # Read microbiome data based on file extension
   if (microbiome_ext == "txt") {
@@ -741,8 +819,14 @@ ezviz <- function(microbiome_data, metadata, level, selected_metadata, top_taxa
     return("The microbiome file is not in a supported format. Please use txt, csv, xls, or xlsx.")
   }
   
+  # — sanitize first column values — 
+  # replace spaces with underscores in whatever the first column is  
+  first_col <- names(microbiome_df)[1]  
+  microbiome_df[[first_col]] <- gsub("\\s+", "_", as.character(microbiome_df[[first_col]]))
   
   
+  microbiome_base <- tools::file_path_sans_ext(basename(microbiome_data))
+  
   
   # Define the levels map with variations
   levels_map <- c(
@@ -783,15 +867,19 @@ ezviz <- function(microbiome_data, metadata, level, selected_metadata, top_taxa
   cleaned_file_ezviz <- ezclean(microbiome_data, metadata, level)
   message("ezclean returned:", cleaned_file_ezviz, "\n")
   
-  # Check if the file exists
+  # 1a) Copy into working dir:
+  cleaned_basename <- basename(cleaned_file_ezviz)
+  file.copy(cleaned_file_ezviz, cleaned_basename, overwrite = TRUE)
+  
+  # 1b) Point the variable at the local copy:
+  cleaned_file_ezviz <- cleaned_basename
+  
   if (!file.exists(cleaned_file_ezviz)) {
     stop("The file returned by ezclean does not exist: ", cleaned_file_ezviz)
   }
-  
-  # Read the Excel file returned by ezclean using openxlsx
-  # Read the Excel file returned by ezclean using openxlsx
   cleaned_data <- openxlsx::read.xlsx(cleaned_file_ezviz, colNames = TRUE, check.names = FALSE)
   
+  
   # Option 1: Prevent conversion by ensuring check.names = FALSE everywhere (preferred)
   
   # OR, Option 2: Conditionally replace dots with spaces:
@@ -1064,14 +1152,16 @@ ezviz <- function(microbiome_data, metadata, level, selected_metadata, top_taxa
   # Write the final data frame to "mbX_cleaning_6.xlsx"
   # Define a mapping for visualization data file names based on level_code
   # Define a mapping for the visualization data file names based on level_code
+  
+  
   vizDataNames <- list(
-    "d__" = "mbX_vizualization_data_domains.xlsx",
-    "p__" = "mbX_vizualization_data_phyla.xlsx",
-    "c__" = "mbX_vizualization_data_classes.xlsx",
-    "o__" = "mbX_vizualization_data_orders.xlsx",
-    "f__" = "mbX_vizualization_data_families.xlsx",
-    "g__" = "mbX_vizualization_data_genera.xlsx",
-    "s__" = "mbX_vizualization_data_species.xlsx"
+    "d__" = paste0("mbX_vizualization_data_domains_",  "_", microbiome_base,  "_","by", "_", selected_metadata , ".xlsx"),
+    "p__" = paste0("mbX_vizualization_data_phyla_", microbiome_base,  "_","by", "_", selected_metadata , ".xlsx"),
+    "c__" = paste0("mbX_vizualization_data_classes_", microbiome_base,  "_","by", "_", selected_metadata , ".xlsx"),
+    "o__" = paste0("mbX_vizualization_data_orders_", microbiome_base,  "_","by", "_", selected_metadata , ".xlsx"),
+    "f__" = paste0("mbX_vizualization_data_families_",microbiome_base,  "_","by", "_", selected_metadata , ".xlsx"),
+    "g__" = paste0("mbX_vizualization_data_genera_", microbiome_base,  "_","by", "_", selected_metadata , ".xlsx"),
+    "s__" = paste0("mbX_vizualization_data_species_", microbiome_base,  "_","by", "_", selected_metadata , ".xlsx")
   )
   
   # Determine the output file name based on level_code
@@ -1161,33 +1251,56 @@ ezviz <- function(microbiome_data, metadata, level, selected_metadata, top_taxa
   # Reshape the data from wide to long format
   # The first column is taxon names; all remaining columns represent samples.
   df_long <- df_clean6 %>%
-    pivot_longer(
-      cols = -1,                      # leave the first column untouched
+    tidyr::pivot_longer(
+      cols = -1,
       names_to = "Sample",
       values_to = "Abundance"
     )
   
-  # Ensure the taxon column is treated as a factor (preserving the original order)
-  taxa_col <- names(df_clean6)[1]
-  df_long[[taxa_col]] <- factor(df_long[[taxa_col]], levels = df_clean6[[taxa_col]])
+  # 2) Build the factor on the taxon column in the same order as they appear in df_clean6[,1]:
+  taxa_col       <- names(df_clean6)[1]
+  # ensure unique taxa so we don't get duplicate factor‐level errors
+  original_levels <- unique(as.character(df_clean6[[taxa_col]]))
+  # (this is a vector of taxa in the order they were in the cleaned table)
+  
+  df_long[[taxa_col]] <- factor(
+    df_long[[taxa_col]],
+    levels = original_levels
+  )
+  
+  # 3) If flip = TRUE, *reverse* those factor levels on df_long[[taxa_col]]:
+  
+  flip_flag <- FALSE
+  if (is.logical(flip) && flip) {
+    flip_flag <- TRUE
+  } else if (is.character(flip) && tolower(flip) == "true") {
+    flip_flag <- TRUE
+  }
+  
+  # Reverse the stacking order if requested:
+  if (flip_flag) {
+    df_long[[taxa_col]] <- factor(
+      df_long[[taxa_col]],
+      levels = rev(levels(df_long[[taxa_col]]))
+    )
+  }
+  
+  
+  
+  
+  # 4) Now df_long[[taxa_col]] has either original_levels (flip = FALSE) or reversed (flip = TRUE).
+  #    Next you can define your palettes and do ggplot on df_long as before:
   
-  # Define the two base palettes
   tab10 <- c("#1f77b4", "#ff7f0e", "#2ca02c", "#d62728", "#9467bd",
              "#8c564b", "#e377c2", "#7f7f7f", "#bcbd22", "#17becf")
   set3 <- c("#8dd3c7", "#ffffb3", "#bebada", "#fb8072", "#80b1d3",
             "#fdb462", "#b3de69", "#fccde5", "#d9d9d9", "#bc80bd")
   
-  # We want 200 colors in total. That means 100 values from each palette.
   tab10_rep <- rep(tab10, length.out = 100)
   set3_rep  <- rep(set3, length.out = 100)
-  
-  # Interleave the two repeated palettes
   custom_colors <- as.vector(rbind(tab10_rep, set3_rep))
   
-  # Calculate the number of distinct taxa (for distinct colors)
   n_colors <- length(unique(df_long[[taxa_col]]))
-  
-  # If needed, subset custom_colors to match the number of taxa
   if(n_colors > length(custom_colors)){
     warning("Number of taxa exceeds custom color length; colors will be recycled.")
     palette_colors <- rep(custom_colors, length.out = n_colors)
@@ -1195,7 +1308,6 @@ ezviz <- function(microbiome_data, metadata, level, selected_metadata, top_taxa
     palette_colors <- custom_colors[1:n_colors]
   }
   
-  # Create the stacked bar plot using the custom colors
   p <- ggplot(df_long, aes(x = Sample, y = Abundance, fill = .data[[taxa_col]])) +
     geom_bar(stat = "identity", position = "stack") +
     scale_fill_manual(values = palette_colors) +
@@ -1219,16 +1331,18 @@ ezviz <- function(microbiome_data, metadata, level, selected_metadata, top_taxa
   
   
   # Define output filename dynamically
+  
+  
   final_names_viz <- list(
-    "d__" = "mbX_viz_domains_or_kingdom",
-    "p__" = "mbX_viz_phylum",
-    "c__" = "mbX_viz_classes",
-    "o__" = "mbX_viz_orders",
-    "f__" = "mbX_viz_families",
-    "g__" = "mbX_viz_genera",
-    "s__" = "mbX_viz_species"
+    "d__" = paste0("mbX_viz_domains_or_kingdoms_", microbiome_base, "_","by", "_", selected_metadata),
+    "p__" = paste0("mbX_viz_phyla_",                     microbiome_base, "_","by", "_", selected_metadata, ".pdf"),
+    "c__" = paste0("mbX_viz_classes_",                    microbiome_base, "_","by", "_", selected_metadata, ".pdf"),
+    "o__" = paste0("mbX_viz_orders_",                     microbiome_base, "_","by", "_", selected_metadata, ".pdf"),
+    "f__" = paste0("mbX_viz_families_",                   microbiome_base, "_","by", "_", selected_metadata, ".pdf"),
+    "g__" = paste0("mbX_viz_genera_",                     microbiome_base, "_","by", "_", selected_metadata, ".pdf"),
+    "s__" = paste0("mbX_viz_species_",                    microbiome_base, "_","by", "_", selected_metadata, ".pdf")
   )
-  output_plot_filename <- paste0(final_names_viz[[ level_code ]], ".pdf")
+  output_plot_filename <- paste0(final_names_viz[[ level_code ]])
   
   
   # Save the plot with DPI = 1800 and dimensions adjusted to avoid cropping
@@ -1279,7 +1393,47 @@ ezviz <- function(microbiome_data, metadata, level, selected_metadata, top_taxa
       #cat("File not found (already deleted or never created):", file, "\n")
     }
   }
+  
+  
+  
+  
+  final_names_viz_dir <- list(
+    "d__" = paste0("mbX_viz_domains_or_kingdoms_", microbiome_base),
+    "p__" = paste0("mbX_viz_phyla_",                     microbiome_base),
+    "c__" = paste0("mbX_viz_classes_",                    microbiome_base),
+    "o__" = paste0("mbX_viz_orders_",                     microbiome_base),
+    "f__" = paste0("mbX_viz_families_",                   microbiome_base),
+    "g__" = paste0("mbX_viz_genera_",                     microbiome_base),
+    "s__" = paste0("mbX_viz_species_",                    microbiome_base)
+  )
+  output_plot_filename_dir <- paste0(final_names_viz_dir[[ level_code ]], ".pdf")
+  
+  
+  viz_folder <- tools::file_path_sans_ext(output_plot_filename_dir)
+  if (!dir.exists(viz_folder)) {
+    dir.create(viz_folder)
+  }
+  
+  # Move both the Excel and the PDF into that new folder
+  #    (outputVizFile is the XLSX, output_plot_filename is the PDF)
+  if (file.exists(outputVizFile)) {
+    file.rename(outputVizFile, file.path(viz_folder, outputVizFile))
+  }
+  if (file.exists(output_plot_filename)) {
+    file.rename(output_plot_filename, file.path(viz_folder, output_plot_filename))
+  }
+  
+  # ── REMOVE THE EZCLEAN OUTPUT FROM WORKING DIRECTORY ──
+  if (exists("cleaned_basename") && file.exists(cleaned_basename)) {
+   file.remove(cleaned_basename)
+}
+  
+  message("ezviz outputs have been saved to: ", viz_folder)
   message("Done with the vizualization, cite us!")
+  message('Please cite us: "Lamichhane, U., & Lourenco, J. (2025). mbX: An R Package for Streamlined Microbiome Analysis. Stats, 8(2), 44."')
+  message("DOI to paper: https://doi.org/10.3390/stats8020044")
+  
+  message("*****PLEASE IGNORE THE FOLLOWING WARNING MESSAGE - In rm(data) : object 'data' not found, warning message!*****")
   
 }
 if (getRversion() >= "2.15.1") {
@@ -1292,6 +1446,471 @@ if (getRversion() >= "2.15.1") {
 #ezviz("level-7.csv", "metadata.txt", "f", "sample_type2", threshold = 1) 
 
 
+#' Statistical Analysis and Visualization of Microbiome Data
+#'
+#' Performs Kruskal_Wallis tests, post_hoc Dunn comparisons, Compact Letter Display (CLD)
+#' summaries, and generates boxplots annotated with CLD letters for taxa abundances
+#' grouped by a chosen metadata variable.
+#'
+#' @param microbiome_data Character; path to the microbiome abundance table (CSV, TSV, XLS, or XLSX).
+#' @param metadata        Character; path to the sample metadata file (CSV, TXT, XLS, or XLSX).
+#' @param level           Character; taxonomic rank to aggregate at (e.g. "genus", "g").
+#' @param selected_metadata Character; name of the categorical metadata column to group by.
+#'
+#' @return Invisibly returns the \code{data.frame} of cleaned sample_taxa abundances used for all analyses.
+#'
+#' @details
+#' This function first calls ezclean to produce a cleaned, merged table of sample IDs,
+#' metadata, and taxa abundances at the requested taxonomic level.  It then:
+#' \enumerate{
+#'   \item Runs Kruskal_Wallis tests on each taxon and writes results with FDR_correction.
+#'   \item Performs Dunns pairwise post_hoc tests (BH_adjusted) for taxa with KW p less than or equal to 0.05.
+#'   \item Computes CLD letters for significantly different groups and writes a summary Excel.
+#'   \item Generates high-resolution (900 dpi) boxplots annotated with CLD letters.
+#' }
+#'
+#' @examples
+#' \dontrun{
+#'   mb  <- system.file("extdata", "microbiome.csv", package = "mbX")
+#'   md  <- system.file("extdata", "metadata.csv",   package = "mbX")
+#'   if (nzchar(mb) && nzchar(md)) {
+#'     ezstat(mb, md, "genus", "Group")
+#'   }
+#' }
+#'
+#' @importFrom stats kruskal.test p.adjust
+#' @importFrom utils read.delim read.csv
+#' @importFrom openxlsx read.xlsx write.xlsx
+#' @importFrom readxl read_excel
+#' @importFrom tools file_ext file_path_sans_ext
+#' @importFrom stats median
+#' @importFrom tibble deframe
+#' @import dplyr
+#' @import rstatix
+#' @import FSA
+#' @import tibble
+#' @import multcompView
+#' @import ggplot2
+#' @export
+ezstat <- function(microbiome_data, metadata, level, selected_metadata) {
+  #
+  #
+  levels_map <- c(
+    "domain"  = "d__", "Domain"  = "d__", "DOMAIN"  = "d__", "D" = "d__", "d" = "d__",
+    "kingdom" = "d__", "Kingdom" = "d__", "KINGDOM" = "d__", "K" = "d__", "k" = "d__",
+    "phylum"  = "p__", "Phylum"  = "p__", "PHYLUM"  = "p__", "P" = "p__", "p" = "p__",
+    "class"   = "c__", "Class"   = "c__", "CLASS"   = "c__", "C" = "c__", "c" = "c__",
+    "order"   = "o__", "Order"   = "o__", "ORDER"   = "o__", "O" = "o__", "o" = "o__",
+    "family"  = "f__", "Family"  = "f__", "FAMILY"  = "f__", "F" = "f__", "f" = "f__",
+    "genera"  = "g__", "genus"   = "g__", "Genera"  = "g__", "GENERA" = "g__", "G" = "g__", "g" = "g__",
+    "species" = "s__", "Species" = "s__", "SPECIES" = "s__", "S" = "s__", "s" = "s__"
+  )
+  tax_code <- levels_map[tolower(level)]
+  if (is.na(tax_code)) {
+    stop("Invalid taxonomic level.  Please choose one of: domain, phylum, class, order, family, genus, or species (or their abbreviations).")
+  }
+  code_to_name <- c(
+    "d__" = "domain",
+    "p__" = "phylum",
+    "c__" = "class",
+    "o__" = "order",
+    "f__" = "family",
+    "g__" = "genus",
+    "s__" = "species"
+  )
+  canonical_level <- code_to_name[[tax_code]]
+  
+  #
+  #  2) Run ezclean() and read its output **in place** (no moving).
+  #
+  cleaned_path <- ezclean(microbiome_data, metadata, level)
+  if (!file.exists(cleaned_path)) {
+    stop("ezclean() did not return a valid file path.  Got: ", cleaned_path)
+  }
+  
+  cleaned_df <- read.xlsx(cleaned_path, colNames = TRUE, check.names = FALSE)
+  
+  #
+  #  3) Load the original metadata (to check selected_metadata is valid & categorical).
+  #
+  meta_ext <- tools::file_ext(metadata)
+  if (meta_ext == "txt") {
+    metadata_df <- utils::read.delim(metadata, sep = "\t", header = TRUE, check.names = FALSE)
+  } else if (meta_ext == "csv") {
+    metadata_df <- read.csv(metadata, header = TRUE, check.names = FALSE)
+  } else if (meta_ext %in% c("xls", "xlsx")) {
+    metadata_df <- readxl::read_excel(metadata, col_names = TRUE) |> as.data.frame(check.names = FALSE)
+  } else {
+    stop("Please check the file format of 'metadata'.")
+  }
+  
+  if (!(selected_metadata %in% colnames(metadata_df))) {
+    stop("'", selected_metadata, "' is not a column of the metadata.")
+  }
+  if (!(is.factor(metadata_df[[selected_metadata]]) || is.character(metadata_df[[selected_metadata]]))) {
+    stop("The selected metadata ('", selected_metadata, "') is not categorical.")
+  }
+  
+  
+  
+  # — sanitize metadata column names & values —
+  # replace spaces in column names
+  colnames(metadata_df) <- gsub("\\s+", "_", colnames(metadata_df))
+  
+  # for any character columns, replace spaces in the values
+  char_cols <- sapply(metadata_df, is.character)
+  metadata_df[ , char_cols] <- lapply(
+    metadata_df[ , char_cols, drop = FALSE],
+    function(x) gsub("\\s+", "_", x)
+  )
+  
+  # for any factor columns, replace spaces in the levels
+  factor_cols <- sapply(metadata_df, is.factor)
+  metadata_df[ , factor_cols] <- lapply(
+    metadata_df[ , factor_cols, drop = FALSE],
+    function(x) factor(gsub("\\s+", "_", as.character(x)))
+  )
+  
+  
+  
+  
+  
+  
+  
+  
+  
+  
+  #
+  #  4) Identify which metadata columns survived in cleaned_df; build subset:
+  #
+  meta_cols_in_cleaned <- intersect(colnames(metadata_df), colnames(cleaned_df))
+  if (!selected_metadata %in% meta_cols_in_cleaned) {
+    stop("After cleaning, '", selected_metadata, "' was not found in the cleaned file.")
+  }
+  
+  #########new line added jul12
+  
+  cleaned_df[[selected_metadata]] <- gsub("-", "_", cleaned_df[[selected_metadata]])
+  
+  #########new line added jul12
+  
+  sample_id_col <- colnames(cleaned_df)[1]
+  taxa_cols     <- setdiff(colnames(cleaned_df), meta_cols_in_cleaned)
+  cols_to_keep  <- c(sample_id_col, selected_metadata, taxa_cols)
+  stat_df       <- cleaned_df[, cols_to_keep, drop = FALSE]
+  
+  #
+  #  5) Create output directory "ezstat_outputs_<canonical_level>_<microbiome_base>"
+  #
+  microbiome_base <- tools::file_path_sans_ext(basename(microbiome_data))
+  dir_name        <- paste0("mbX_stats_", canonical_level, "_", microbiome_base)
+  if (!dir.exists(dir_name)) dir.create(dir_name)
+  
+  #
+  #  6) Write "mbX_ezstat_1.xlsx" 
+  #
+  out1 <- file.path(dir_name, "mbX_ezstat_1.xlsx")
+  openxlsx::write.xlsx(stat_df, file = out1, rowNames = FALSE)
+  message("ezstat wrote: ", out1)
+  
+  #
+  #  7) Kruskal_Wallis on taxa columns (3rd onward) grouped by selected_metadata
+  #
+  
+  df_stat <- stat_df
+  if (is.character(df_stat[[selected_metadata]])) {
+    df_stat[[selected_metadata]] <- factor(df_stat[[selected_metadata]])
+  }
+  
+  taxa_names <- names(df_stat)[3:ncol(df_stat)]
+  test_stats <- numeric(length(taxa_names))
+  p_values   <- numeric(length(taxa_names))
+  
+  for (i in seq_along(taxa_names)) {
+    this_taxon <- taxa_names[i]
+    vals  <- df_stat[[this_taxon]]
+    grps  <- df_stat[[selected_metadata]]
+    if (length(unique(grps[!is.na(vals)])) < 2) {
+      test_stats[i] <- NA
+      p_values[i]   <- NA
+    } else {
+      kt            <- kruskal.test(vals ~ grps)
+      test_stats[i] <- as.numeric(kt$statistic)
+      p_values[i]   <- kt$p.value
+    }
+  }
+  FDR_pvals <- p.adjust(p_values, method = "fdr")
+  
+    results_kw <- data.frame(
+    taxa           = taxa_names,
+    teststat_value = test_stats,
+    P_value        = p_values,
+    FDR_P_value    = FDR_pvals,
+    stringsAsFactors = FALSE
+  )
+
+  # ── Drop the FDR_P_value column *only* from the file you write ──
+  results_kw_to_save <- results_kw[, c("taxa","teststat_value","P_value")]
+
+  out_kw <- file.path(
+    dir_name,
+    paste0("ezstat_KW_tests_", canonical_level, "_","by", "_", selected_metadata, ".xlsx")
+  )
+  openxlsx::write.xlsx(results_kw_to_save, file = out_kw, rowNames = FALSE)
+  message(" ezstat wrote: ", out_kw)
+
+  
+  #
+  #  9) Pairwise Dunns comparisons for any taxon with KW P_value less than or equals to 0.05
+  #
+  
+  kw_df    <- openxlsx::read.xlsx(out_kw, colNames = TRUE, check.names = FALSE)
+  sig_taxa <- kw_df %>%
+    filter(!is.na(P_value) & P_value <= 0.05) %>%
+    pull(taxa)
+  
+  dunn_list <- list()
+  for (tax in sig_taxa) {
+    temp <- df_stat %>%
+      select(!!sym(selected_metadata), !!sym(tax)) %>%
+      rename(Group = !!sym(selected_metadata), Value = !!sym(tax)) %>%
+      mutate(Value = as.numeric(Value)) %>%
+      filter(!is.na(Value))
+    
+    if (n_distinct(temp$Group) < 2) next
+    
+    dunn_res <- temp %>%
+      dunn_test(Value ~ Group, p.adjust.method = "BH") %>%
+      rename(
+        Comparison1  = group1,
+        Comparison2  = group2,
+        statistic    = statistic,
+        P_value      = p,
+        FDR_P_value  = p.adj
+      ) %>%
+      mutate(
+        Comparison = paste0(Comparison1, "_vs_", Comparison2),
+        taxa       = tax
+      ) %>%
+      select(taxa, Comparison, statistic, P_value, FDR_P_value)
+    
+    dunn_list[[tax]] <- dunn_res
+  }
+  
+  pairwise_df <- bind_rows(dunn_list)
+  out_pw <- file.path(
+    dir_name,
+    paste0("ezstat_pairwise_", canonical_level, "_","by", "_", selected_metadata, ".xlsx")
+  )
+  openxlsx::write.xlsx(pairwise_df, file = out_pw, rowNames = FALSE)
+  message("ezstat wrote: ", out_pw)
+  
+  #
+  # 10) CLD_Summary: for each taxon in sig_taxa, run FSA::dunnTest( ) to build a p vector
+  #
+  
+  
+  cld_list <- list()
+  for (tax in sig_taxa) {
+    temp_df <- df_stat %>%
+      select(!!sym(selected_metadata), !!sym(tax)) %>%
+      rename(Group = !!sym(selected_metadata), Value = !!sym(tax)) %>%
+      mutate(Value = as.numeric(Value)) %>%
+      filter(!is.na(Value))
+    
+    if (n_distinct(temp_df$Group) < 2) next
+    
+    # 10.1) Compute per_Group median & mean
+    summary_stats <- temp_df %>%
+      group_by(Group) %>%
+      summarise(
+        Median = median(Value, na.rm = TRUE),
+        Mean   = mean(Value,   na.rm = TRUE),
+        .groups = "drop"
+      )
+    
+        # 10.2) Read in your pairwise-Dunn results and extract the BH‐adjusted p‐values
+    pw_file <- file.path(
+      dir_name,
+      paste0("ezstat_pairwise_", canonical_level, "_","by", "_", selected_metadata, ".xlsx")
+    )
+    pw_df <- openxlsx::read.xlsx(pw_file, colNames = TRUE, check.names = FALSE) %>%
+             mutate(Comparison = gsub("\\s+", "", Comparison))
+
+    # For this taxon, grab its comparisons and FDR_P_value
+    tax_pw <- pw_df %>% filter(taxa == tax)
+	pval_vec <- setNames(tax_pw$FDR_P_value, gsub("_vs_", "-", tax_pw$Comparison))
+
+    groups_present <- sort(unique(temp_df$Group))
+    n_groups       <- length(groups_present)
+
+    if (n_groups == 2) {
+      # two‐group special case: only one adjusted p‐value in pval_vec
+      padj    <- unname(pval_vec)[1]
+      letters <- if (padj <= 0.05) c("a","b") else c("a","a")
+      cld_df <- data.frame(
+        Group = groups_present,
+        CLD   = letters,
+        stringsAsFactors = FALSE
+      )
+    } else {
+      needed <- choose(n_groups, 2)
+      if (length(pval_vec) == needed) {
+        cld_out <- multcompLetters(pval_vec)
+        cld_df   <- data.frame(
+          Group = names(cld_out$Letters),
+          CLD   = unname(cld_out$Letters),
+          stringsAsFactors = FALSE
+        )
+      } else {
+        # fallback if your pairwise file is missing any comparisons
+        cld_df <- data.frame(
+          Group = groups_present,
+          CLD   = rep("a", n_groups),
+          stringsAsFactors = FALSE
+        )
+      }
+    }
+
+    
+    
+    
+    # 10.5) Merge summary_stats and CLD letters, then reorder/rename
+    merged_cld <- summary_stats %>%
+      left_join(cld_df, by = "Group") %>%
+      mutate(
+        taxa                 = tax,
+        !!selected_metadata  := Group
+      ) %>%
+      select(
+        taxa,
+        !!selected_metadata,
+        Median,
+        Mean,
+        Group = CLD
+      )
+    
+    cld_list[[tax]] <- merged_cld
+  }
+  
+  cld_summary_df <- bind_rows(cld_list)
+  out_cld <- file.path(
+    dir_name,
+    paste0("ezstat_CLD_Summary_", canonical_level, "_", "by", "_", selected_metadata, ".xlsx")
+  )
+  openxlsx::write.xlsx(cld_summary_df, file = out_cld, rowNames = FALSE)
+  message("ezstat wrote: ", out_cld)
+  
+  #
+  # 11) Generate Boxplots for each taxon with KW P_value less than or equals to  0.05, annotated with CLD letters
+  #
+  
+  # Create a subdirectory "Boxplots_<selected_metadata>" inside our ezstat_outputs folder
+  box_dir <- file.path(dir_name, paste0("Boxplots_", selected_metadata))
+  if (!dir.exists(box_dir)) dir.create(box_dir)
+  
+  # Re_read the Kruskal_Wallis results to identify significant taxa
+  kw_df    <- openxlsx::read.xlsx(out_kw, colNames = TRUE, check.names = FALSE)
+  sig_taxa <- kw_df %>%
+    filter(!is.na(P_value) & P_value <= 0.05) %>%
+    pull(taxa)
+  
+  # Re_read the CLD summary table so we know which letter goes to which group
+  cld_summary_df <- openxlsx::read.xlsx(out_cld, colNames = TRUE, check.names = FALSE)
+  # cld_summary_df has columns: taxa, <selected_metadata>, Median, Mean, Group (where Group is the CLD letter)
+  
+  for (tax in sig_taxa) {
+    # 11.1) Build a small data.frame of (Group, Value) for this taxon
+    temp_df <- df_stat %>%
+      select(!!sym(selected_metadata), !!sym(tax)) %>%
+      rename(Group = !!sym(selected_metadata), Value = !!sym(tax)) %>%
+      mutate(Value = as.numeric(Value)) %>%
+      filter(!is.na(Value))
+    
+    # Skip if fewer than two distinct groups remain
+    if (n_distinct(temp_df$Group) < 2) next
+    
+    # 11.2) Extract the CLD letters for this taxon
+    letter_df <- cld_summary_df %>%
+      filter(taxa == tax) %>%
+      select(!!sym(selected_metadata), Group) %>%
+      rename(Group = !!sym(selected_metadata), Letter = Group)
+    # Now `letter_df` has columns: Group (the category) and Letter (the CLD letter)
+    
+    # 11.3) Compute a reasonable y_position for each group's letter (a bit above the boxplot)
+    y_pos_df <- temp_df %>%
+      group_by(Group) %>%
+      summarise(y_max = max(Value, na.rm = TRUE), .groups = "drop") %>%
+      mutate(y = ifelse(y_max == 0, 0.001, y_max + 0.05 * y_max))
+    # If the groups max is 0, we nudge it to 0.001 so the letter is visible
+    
+    # 11.4) Combine letter positions
+    letter_pos <- letter_df %>%
+      left_join(y_pos_df %>% select(Group, y), by = "Group")
+    
+    # 11.5) Build the boxplot
+    p <- ggplot(temp_df, aes(x = Group, y = Value)) +
+      geom_boxplot(outlier.shape = NA) +
+      geom_jitter(width = 0.2, alpha = 0.5, size = 0.8) +
+      geom_text(
+        data = letter_pos,
+        aes(x = Group, y = y, label = Letter),
+        vjust = 0,
+        size = 5,
+        fontface = "bold"
+      ) +
+      labs(
+        title = paste0("Abundance of ", tax, " by ", selected_metadata),
+        x = selected_metadata,
+        y = "Relative Abundance"
+      ) +
+      theme_bw() +
+      theme(
+        plot.title  = element_text(hjust = 0.5, face = "bold", size = 14),
+        axis.text.x = element_text(angle = 45, hjust = 1),
+        axis.title = element_text(face = "bold")
+      )
+    
+    # 11.6) Save as a 900 DPI PNG file
+    out_png <- file.path(box_dir, paste0(tax, ".png"))
+    ggsave(
+      filename = out_png,
+      plot     = p,
+      dpi      = 900,
+      width    = 6,
+      height   = 4,
+      units    = "in"
+    )
+  }
+  
+  message("ezstat wrote boxplots to: ", box_dir)
+  message("All ezstat outputs are in: ", dir_name)
+  
+  message('Please cite us: "Lamichhane, U., & Lourenco, J. (2025). mbX: An R Package for Streamlined Microbiome Analysis. Stats, 8(2), 44."')
+  message("DOI to paper: https://doi.org/10.3390/stats8020044")
+  
+  
+  #if (getRversion() >= "2.15.1") {
+    #utils::globalVariables(c(
+     # "P_value", "taxa", "group1", "group2", "statistic",
+      #"p.adj", "Comparison1", "Comparison2", "Comparison",
+      #"FDR_P_value", "Group", "P.adj", "Median", "Mean",
+      #"CLD", "y_max", "y", "Letter", ":="
+    #))
+  #}
+  
+  #remove that file
+  if (file.exists(out1)) file.remove(out1)
+  
+  return(invisible(stat_df))
+}
+
+
+
+
+#ezstat("microbiome10.csv", "metadata10.csv", "g", "check")
+#ezviz("microbiome10.csv", "metadata10.csv", "g", "check", top_taxa =10)
+#ezclean("microbiome10.csv", "metadata10.csv", "g")
 
 
 
